///////////////////////////////////////////////////////////////////////////
// Canon EOS M Shutter-Emulation on Arduino (Nano / Pro Mini, / ...) V3.1
//
//       10.4.2020                       Erwin Lotter
//
//#define EOS_M
//#define EOS_M3
#define EOS_M100    // getestet fr die M100
// Der Shutter der M10 ist vmtl. identisch zu dem in der M100 verbauten!

#define SERIAL_DEBUG  115200  // undef oder Baud rate
//#define DEBUG_SLOW    // 16x langsamer, Schalten mit LED (LED_BUILTIN, meist D13) anzeigen
//#define DEBUG_BLINK    // Sequenzende mit LED anzeigen
//
// The shutter-emulation code is based on the work reported by
//  Hendrik Beijeman (http://www.phym.nl/wp/?p=367).
//
// Anders als der Code fr die 600D und 1100D kann der Emulator fr die EOS M als
//  gewhnlicher Sketch statt als Boot-Code ausgefhrt werden. (Einschalten geht auch ohne
//  Shutter: LS1 und LS2 mssen Hi sein, was auch offen oder wenn der Arduino noch
//  nicht bereit ist, der Fall ist!
//
// Geeignete Hardware ist z.B. der Pro Mini sowohl in 5V/16MHz als auch in
//  3.3V/8MHz Ausfhrung.
//
// Die EOS M3 erwartet Low an Opt1. Wenn dies mit einem Pull-Down 
//  sichergestellt wird, kann der Emulator als gewhnlicher Sketch geladen
//  und mit der Spannung der Coil 1 versorgt werden.
//
// Die EOS M10/M100 erwarten High an Opt1. Damit kann der Emulator einfach
//  mit der Spannung der Coil 1 versorgt werden, auch wenn diese spt kommt
//  und als gewhnlicher Sketch geladen werden.
//

// Mgliche Ergebnisse eines Trigger-Ereignisses (Interrupts)
#define evCoilStart  1
#define evCoilStop   2
#define evMotorOn    3
#define evNone       0

struct sAction {
  unsigned long time;
  unsigned int  pin;
  unsigned int  value;
};
#define ms *1000L
#define STOP_VAL  2
#define BLINK_VAL   3
#define MOTOR_VAL   4
#define STOP_ACTION  {0, 0, STOP_VAL}
#define BLINK_ACTION  {0, 0, BLINK_VAL}   // toggled LED, wenn DEBUG_BLINK definiert ist: Nicht als 1. Eintrag!
#if !defined(EOS_M100)
# define MOTOR_ACTION  {0, 0, MOTOR_VAL}   // aktiviert Motor-Interrupt
#endif

#if defined(SERIAL_DEBUG)
#define PRINT(a)    Serial.print(a)
#define PRINTLN(a)  Serial.println(a)
void print(long usecs) {
  char bf[4];
  int millsecs = usecs/1000;
  bf[0] = millsecs/10 + '0';
  bf[1] = millsecs%10 + '0';
  bf[2] = ' ';
  bf[3] = 0;
  PRINT(bf);
}
#else
#define PRINT(a)
#define PRINTLN(a)
void print(long) {}
#endif


#ifdef EOS_M
// 
// Der Shutter der EOS M ist im Ruhezustand offen und selbsthaltend. 
// Zu Beginn einer Aufnahme werden der Motor und die Coil aktiviert, um den Shutter dadurch
//  scharf zu machen, dass er ber den selbsthaltenden Offen-Punkt hinaus bewegt und bis
//  zu derer Deaktivierung von der Coil offen gehalten wird. LS1 prft dabei, wann die
//  Mechanik 'aufgezogen' ist und der Motor wieder abgeschaltet werden muss.
// Zum Ende der Aufnahme wird die Coil abgeschaltet und LS2 prft das Schlieen der Blades.
// Danach ist die Aufnahme abgeschlossen und der Verschluss wird wieder geffnet. Dabei stoppt
//  der Motor sobald der selbsthaltende Offen-Punkt erreicht ist, was LS1=H anzeigt.
// Das Timing (in Millisekunden) ist fr alle Belichtungen gleichartig (LS1==OPT1, LS2==OPT2): 
// 
//  (1.) 0    M1: H
//       5    Coil: L
//       7    LS1: L        (M1 und Coil beide geschaltet!)
//      21    LS1: H,  M1: L
//    (2.)    T+x    Coil: H
//        5    LS2: L
//        7    LS2: H
//        8    LS2: L
//       16    LS2: H
//    (3.) 70+T  M1:H
//          0    LS2:L
//         13        LS1:L
//         21    LS2:H
//         27    LS2:L
//         34    LS2:H
//         42        LS1:H
// 
// Es gibt drei Referenz-Zeitpunkte:
// 1.) Bei t=0 startet der Motor. Da dies auch bei (3) passiert, muss gewartet werden, ob Coil LOW folgt.
//     Die folgenden LS1 Signale knnen dann auf Coil LOW bezogen werden.
// 2.) Das Ende der Aufnahme wird an der Abschaltung der Coil erkannt und die LS2-Signale werden generiert.
// 3.) Der Verschluss wird wieder geffnet. Dazu luft der Motor an, ohne dass die Coil aktiviert wird.
//     Bei kurzer Belichtung knnte diese Sequenz auch als Fortsetzung von 2.) implementiert werden,
//       im Bulb-Mode bleibt der Shutter aber noch fr bis zu 6 Sekunden geschlossen - auch wenn
//      "Rauschreduzierung bei Langzeitaufnahmen" abgeschaltet ist.
//     Das zeitgleich erfolgende LS2:L kann um 5 ms verzgert werden, bis klar ist, ob Coil:L folgt oder nicht.
// 
// Anschluss-Defs.:
//  Int. Bez.| Arduino-D_n (2+3 sind interruptfhig) // Shutter-Pin
#define MOTOR     2     // Pin 15+16  Motor switch (active high)
#define COIL      3     // Pin 8+9  Coil switch (active low), Pull-Up notwendig
#define OPT1      5     // Pin 1:   Opto-feedback near motor (LS1)
#define OPT2      6     // Pin 10:  Opto-feedback from shutter coil
                        // Pin 6+7: Coil Power
                        // Pin 4+11: Gnd
// Alle Anschlsse gehen ber 2k Schutzwiderstand, so dass auch der 5V-Betrieb 
//  des Arduino mglich ist.

#define COIL_ACTIVE   LOW
#define COIL_TRIGGER  CHANGE
#define MOTOR_ACTIVE  HIGH
#define MOTOR_TRIGGER RISING

//#define MOTOR_GATE  // Motor-Signal nur nach Auslsung akzeptieren: UNVOLLSTNDIG!

// Startwerte:
#define OPT2_SUP HIGH
#define OPT1_SUP HIGH

sAction coilStartAction[] = {      // Coil aktiv (kommt ca. 4 ms nach Motor!)
  { 0 ms, OPT2, HIGH},    // Motor wurde zuerst aktiv und hat evtl. OPT2 LOW gesetzt: Jetzt korrigieren!
  { 3 ms, OPT1, LOW},//2
  {17 ms, OPT1, HIGH},//16
  STOP_ACTION,
};

sAction coilStopAction[] = {      // Coil inaktiv
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},
  {15 ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

#ifdef MOTOR_GATE
#define MOTOR_DELAY 0
#else
#define MOTOR_DELAY 5
#endif

 sAction motorAction[] = {      // Motor an: Sequenz fr das Aufziehen nach der Aufnahme.
  { MOTOR_DELAY ms, OPT2, LOW},      // kommt im Original sofort, hier 5 ms verzgert, falls auch noch Coil low wird: dann ist Auslse-Start, bei dem der Motor auch aktieviert wird.
  {13 ms, OPT1, LOW},
  {21 ms, OPT2, HIGH},
  {27 ms, OPT2, LOW},
  {34 ms, OPT2, HIGH},
  {42 ms, OPT1, HIGH},
  STOP_ACTION,
};
#endif

#ifdef EOS_M3
// 
// Die Shutter der EOS M3++ sind im Ruhezustand offen und selbsthaltend. 
// Zu Beginn einer Aufnahme werden der Motor und die Coil1 aktiviert, um den Shutter dadurch
//  scharf zu machen, dass er ber den selbsthaltenden Offen-Punkt hinaus bewegt und bis
//  zu deren Deaktivierung von der Coil1 offen gehalten wird. 
// OPT2 == Hi zeigt an, dass der Shutter offen ist. 
// Die Belichtung beginnt elektronisch, etwa 10 ms nachdem der Motor den Shutter aufgezogen hat,
//  was an OPT1 = High erkannt wird.
// 
// a) Festzeit-Belichtung:
//  Zum Ende der Aufnahme wird die Coil abgeschaltet und OPT2 zeigt das Schlieen der Blades
//  mit Low-High-Low (und, nachdem die OPT2-Power abgeschaltet wurde, wieder High) an.
//  Danach zieht der Motor den Shutter wieder auf, bis Opt1 ffnet (low wird).
//  (OPT1 wird dann sofort wieder Low, vmtl. weil die OPT1-Pwr abgeschaltet wird.)
//  Etwa 130 ms spter schiebt die Coil2 den Shutter wieder auf, was von OPT2
//  mit High-Low-High angezeigt wird.
//  Danach ist die Aufnahme abgeschlossen und der Verschluss wieder selbsthaltend geffnet.
//  Das Timing (in Millisekunden) ist fr alle Belichtungszeiten gleichartig.
// 
// b) Bulb-Mode:
//  Hier luft der Motor schon an, bevor Coil1 lst. Nach OPT1 = Low luft der Motor noch
//  weiter, bis OPT1 wieder High wird. Erst jetzt lst Coil1 und OPT2 zeigt mit Low-High-Low
//  an, dass der Sutter schliet. Nach einer Pause von ca. 280 ms startet dann das 
//  Wiederaufziehen des Shutters wie bei der Festzeit-Belichtung, wobei die Pause vor der
//  Coil2-Aktion ca. 200 statt 120 ms dauert und von der Lnge her schwankt.
//  Deshalb muss im Bulb-Mode der Motor-Interrupt verwendet werden:
//  1.) Gleich nach Beginn der Aufnahme (nach Opt1_H) wird er durch das in die
//      coilStartAction Tabelle eingefgte MOTOR_ACTION Makro aktiviert. Hinter diesem 
//      folgt als Wartepunkt M_WAIT ein (unsinniger) Befehl, der zeitlich nicht erreicht werden kann.
//      Wenn der Motor-Interrupt vor C1_H auslst, luft eine Bulb-Aufnahme und das Timing
//      wird angepasst, indem das angepasste Timing in coilStopBulbAction gewhlt wird.
//      Wenn kein Motor-Interrupt vor C1_H ausgelst wird, was daran erkannt wird, dass die
//      Ausfhrung bei M_WAIT hngt, wird der Motor-Interrupt wieder deaktiviert.
//  2.) Nachdem C1_H die Abarbeitung von coilStopBulbAction angestoen hat, muss, sobald die 
//      Opt2 H-L-H Sequenz erledigt ist, wieder auf den Motor gewartet werden, weil die
//      Wartezeit nicht fest ist. Dazu folgt in coilStopBulbAction jetzt ein MOTOR_ACTION Makro,
//      gefolgt vom Wartepunkt CS_WAIT. An dem erkennt die Ablauflogik, ob der Motor_Interrupt 
//      von hier aktiviert wurde oder in der coilStartAction. (CS_WAIT ist eigentlich unntig,
//      auch auf M_WAIT geprft werden knnte.)
//      
// Bei der M3++ ist das Motorsignal gepulst. Der Interrupt muss deshalb nach 
//  Auftreten deaktiviert werden!
// Der Motor-Interrupt lst sofort nach Aktivierung aus, auch wenn das Motorsignal Low ist!
//  Es ist unklar warum. Zur Unterdrckung wird deshalb erst der zweite Motor-Interrupt 
//  ausgewertet, was kein Problem ist, da das Motorsignal gepulst ist.
// 
// Die Wartezeit bis zum abschlieenden Einrasten des Shutters durch Coil2 ist fr Bulb
//  ebenfalls verschieden, aber konstant, so dass sie in coilFinishBulbAction fest kodiert 
//  werden kann.
// 
// Anschluss-Defs.:
//  Int. Bez.| Arduino-D_n (2+3 sind interruptfhig) // M3 Shutter-Pin |  M10,M100 Shutter-Pin
#define MOTOR     2     // Pin 15+16 | 3+4:  Motor Pwr (6V)
#define COIL      3     // Pin  8 | 12:  Coil1 switch (active low), Pull-Up notwendig
#define OPT1      5     // Pin  1 | 6:   Opto-feedback motor rotation
#define OPT2      6     // Pin 10 | 15:  Opto-feedback shutter curtain
//            Vcc/Raw?  // Pin  9 | 13:  Coil1 Power (4.8V, permanent, 47 Ohm)
//                Gnd   // Pin 11 | 14:  Opt2 Gnd
// Alle Anschlsse gehen ber Schutzwiderstand oder Diode, so dass auch der 5V-Betrieb 
//  des Arduino mglich ist.

#define COIL_ACTIVE   LOW
#define COIL_TRIGGER  CHANGE
#define MOTOR_TRIGGER RISING

//#define MOTOR_GATE  // Motor-Signal nur nach Auslsung akzeptieren: UNVOLLSTNDIG!

// Startwerte
#define OPT1_SUP LOW
#define OPT2_SUP HIGH

sAction coilStartAction[] = {      // Coil aktiv
  { 0 ms, OPT2, HIGH},    //
  { 0 ms, OPT1, LOW},
  {22 ms, OPT1, HIGH},    // (stoppt Motor)
  {25 ms, OPT1, HIGH},    // warten bis Motor sicher aus, dann Motor-Int. freigeben:
  MOTOR_ACTION,
  STOP_ACTION,
};

#define csaO 185
sAction coilStopAction[] = {      // Coil inaktiv
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},         // (dann lo lassen, auch wenn OPwr2 aus ist)
  {64 ms, OPT1, LOW},         // Motor stoppen
  //{(csaO) ms, OPT2, LOW},     // Coil2 schiebt den Curtain in die selbsthaltende Position
  {(csaO+6) ms, OPT2, HIGH},
  {(csaO+9) ms, OPT2, LOW},
  {(csaO+11) ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

sAction motorAction[] = {      // Motor startet whrend Belichtung (nur im Bulb-Mode!)
  {58 ms, OPT1, LOW},           // fast aufgezogen
  {80 ms, OPT1, HIGH},          // aufgezogen, lst C1_L aus
  // jetzt warten auf C1_L: noch kein STOP_ACTION, sonst geht actionPtr verloren
  //  denn an actionPtr==motorAction+2 wird Bulb erkannt (also Motor vor c1_H)!
#define M_WAIT (motorAction + 2)
  {999 ms, OPT1, HIGH},          
  STOP_ACTION,
};

sAction coilStopBulbAction[] = {      // Coil inaktiv fr Bulb
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},
  MOTOR_ACTION,
#define CS_WAIT (coilStopBulbAction + 4)
  {8888 ms, OPT2, LOW},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

sAction coilFinishBulbAction[] = {      // Motor startet im Bulb-Mode zum zweiten Mal (nach der Belichtung)
  {55 ms, OPT1, LOW},           // Motor hat Ziel erreicht
  {(55+202) ms, OPT2, HIGH},    // Coil2 starten und rastet Shutter ein
  {(55+202+3) ms, OPT2, LOW},
  {(55+202+5) ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

#endif

#if defined(EOS_M100)
// 
// Der Shutter der EOS M10/M100 ist im Ruhezustand offen und selbsthaltend. 
// Zu Beginn einer Aufnahme werden der Motor und die Coil1 aktiviert, um den Shutter dadurch
//  scharf zu machen, dass er ber den selbsthaltenden Offen-Punkt hinaus bewegt und bis
//  zu deren Deaktivierung von der Coil1 offen gehalten wird. 
// OPT2 == Hi zeigt an, dass der Shutter offen ist. 
// Die Belichtung beginnt elektronisch, etwa 10 ms nachdem der Motor den Shutter aufgezogen hat,
//  was an OPT1 = Lo erkannt wird.
// 
//  Zum Ende der Aufnahme wird die Coil abgeschaltet und OPT2 zeigt das Schlieen der Blades
//  mit Low-High-Low (und, nachdem die OPT2-Power abgeschaltet wurde, wieder High) an.
//  Danach zieht der Motor den Shutter wieder auf, bis Opt1 ffnet (hi wird).
//  Etwa 130 ms spter schiebt die Coil2 den Shutter wieder auf, was von OPT2
//  mit High-Low-High angezeigt wird.
//  Danach ist die Aufnahme abgeschlossen und der Verschluss wieder selbsthaltend geffnet.
// 
// Anders als bei der M3 wird diese Sequenz auch im Bulb-Mode ausgefhrt.
// Deshalb muss das Motorsignal fr die Emulation nicht ausgewertet werden.
// (Die Pause von ber 2s, die die M100 beim Drcken des Auslsers im Bulb-Mode
//  manchmal macht, hat nichts mit der Emulation zu tun, sie tritt auch mit Shutter auf.)
// 
// Anschluss-Defs.:
//  Int. Bez.| Arduino-D_n (2+3 sind interruptfhig) // M3 Shutter-Pin |  M10,M100 Shutter-Pin
//#define MOTOR     2     // Pin 15+16 | 3+4:  Motor Pwr (6V)
#define COIL      3     // Pin  8 | 12:  Coil1 switch (active low), Pull-Up notwendig
#define OPT1      5     // Pin  1 | 6:   Opto-feedback motor rotation
#define OPT2      6     // Pin 10 | 15:  Opto-feedback shutter curtain
//            Vcc/Raw?  // Pin  9 | 13:  Coil1 Power (4.8V, permanent, 47 Ohm)
//                Gnd   // Pin 11 | 14:  Opt2 Gnd
// Alle Anschlsse gehen ber Schutzwiderstand oder Diode, so dass auch der 5V-Betrieb 
//  des Arduino mglich ist.

#define COIL_ACTIVE   LOW
#define COIL_TRIGGER  CHANGE

// Startwerte
#define OPT1_SUP HIGH   // wie M, umgekehrt wie M3
#define OPT2_SUP HIGH

sAction coilStartAction[] = {      // Coil aktiv
  { 0 ms, OPT2, HIGH},
  { 0 ms, OPT1, HIGH},
  {20 ms, OPT1, LOW},     // (stoppt Motor)
  {25 ms, OPT1, LOW},     // warten bis Motor sicher aus, dann Motor-Int. freigeben:
  STOP_ACTION,
};

#define csaO 126    // Delay bis Coil2 aktiviert wird
sAction coilStopAction[] = {      // Coil inaktiv
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},         // (dann lo lassen, auch wenn OPwr2 aus ist)
  {64 ms, OPT1, HIGH},         // Motor stoppen
  {(csaO+6) ms, OPT2, HIGH},     // Coil2 schiebt den Curtain in die selbsthaltende Position
  {(csaO+9) ms, OPT2, LOW},
  {(csaO+11) ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

#endif
////////////////////////////////////////////////////////////////////////////////////////

sAction* actionPtr;
volatile int triggerEvent;    // Flag fr Trigger-Flanken
unsigned long timingZero;   // Start-Zeit
//int motorSuppress;
void motor_interrupt();
void coil_interrupt();
void SetPin(int pin, int val);

void ToggleLED() 
{
  digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));   // LED umschalten
}
#ifdef DEBUG_SLOW
#define ToggleDebugLED()  ToggleLED()
#define DebugSlowdown(t)  ((t) >> 6)        // /64: 10ms -> 0.64s
#else
#define ToggleDebugLED()
#define DebugSlowdown(t)  (t)
#endif

#ifdef DEBUG_BLINK
#define BlinkLED()  ToggleLED()
#else
#define BlinkLED()
#endif

#include <avr/sleep.h>

// Ablaufschema
// -------------
// Auerhalb der drei Pulsgruppen befindet sich der Prozessor im Standby,
//  aus der ihn eine Flanke von COIL oder MOTOR per Interrupt weckt.
//
// Bis zum Ende der aktuellen Pulsgruppe wird loop() durchlaufen und geprft,
//  ob die in actionPtr->time vermerkte Zeit erreicht ist. 
//  Wenn ja, wird die hinterlegte Aktion ausgefhrt und actionPtr erhht.
//
void setup() {
  //
  // Ruhezustand der emulierten Pins (COIL mit Pullup!):
#ifndef EOS_M100
  pinMode(MOTOR, INPUT);
#endif
  pinMode(COIL,  INPUT_PULLUP);
  pinMode(OPT1,  OUTPUT);
  pinMode(OPT2,  OUTPUT);
#if defined(DEBUG_SLOW) || defined(DEBUG_BLINK)
  pinMode(LED_BUILTIN, OUTPUT);
#endif

  triggerEvent = evNone;
  actionPtr = 0;
  SetPin(  OPT1, OPT1_SUP);    // (ist zwar Hi, aber nur wegen OPwr1=Low)
  SetPin(  OPT2, OPT2_SUP);

  //motorSuppress = 0;
#if defined(EOS_M)
  attachInterrupt(digitalPinToInterrupt(MOTOR), motor_interrupt, MOTOR_TRIGGER);
#endif
  attachInterrupt(digitalPinToInterrupt(COIL),  coil_interrupt,  COIL_TRIGGER);

#ifdef SERIAL_DEBUG
  Serial.begin(SERIAL_DEBUG);
#endif 
}

void loop() {
  //
  // Ablauf eines Ereignisses:
  // --------------------------
  // In den Interrupt-Handlern wird in triggerEvent hinterlegt, welches Ereignis auftrat. 
  // Passend dazu wird hier in actionPtr ein Zeiger auf die Signalsequenz hinterlegt.
  // Anschlieend wird diese Sequenz abgearbeitet und abschlieend der Sleep-Mode aktiviert, aus
  //  dem der Prozessor erst durch den nchsten Interrupt wieder geweckt wird.
  //
  if (triggerEvent != evNone) {        // neuer Interrupt?
    timingZero = micros();             //    Zeitpunkt festhalten
    if (triggerEvent == evCoilStart) { // Start-Flanke Coil:
      actionPtr = coilStartAction;
      PRINTLN("c1_L");
    }
    else if (triggerEvent == evCoilStop) {        // Stopp-Flanke (Coil inaktiv):
#ifdef EOS_M3
      // Fr eine eventuelle Bulb-Auslsung wurde der Motor-Interupt aktiviert.
      // Falls er nicht ausgelst wurde, muss er noch deaktiviert werden:
      // 
      detachInterrupt(digitalPinToInterrupt(MOTOR));  
      //
      // Im Bulb-Mode der M3 wird der Shutter viel spter aufgezogen.
      // Deshalb anderes Timing vorgeben, wenn zuvor ein Motor-Interrupt kam.
      // Das wird daran erkannt, dass die Ausfhrung auf einer Warte-Anweisung
      //  steht
      //
      if (actionPtr == M_WAIT) {   // warten nach Motor-Interrupt?
        actionPtr = coilStopBulbAction;            //  weiter fr Bulb-Abschluss
        PRINTLN("c1_H_StopBulb");
      }
      else 
#endif
      {
        actionPtr = coilStopAction;
        PRINTLN("c1_H_Stop");
      }
    }
#ifndef EOS_M100
    else if (triggerEvent == evMotorOn) {        // Motor-Start (vor c1_H oder nach c1_H):
#ifdef EOS_M3
      if (actionPtr == CS_WAIT) {                // zuletzt Motor-Interrupt in ...?
        //(Alternativ knnte hier auch auf (actionPtr != M_WAIT) geprft werden!)
        actionPtr = coilFinishBulbAction;            //  weiter fr Bulb-Abschluss
        PRINTLN("Motor_H_StopBulb");
      }
      else 
#endif      
      {
        PRINTLN("Motor_H");
        actionPtr = motorAction;
      }
    }
#endif      
    triggerEvent = evNone;
  }
  //
  // Wenn das Ende einer Sequenz erreicht wurde, den Prozessor schlafen legen,
  //  andernfalls in der Loop() weitermachen:
  //
  if (actionPtr == 0) {
    //set_sleep_mode(SLEEP_MODE_PWR_DOWN); // den tiefsten Schlaf auswhlen PWR_DOWN
    set_sleep_mode(SLEEP_MODE_STANDBY); // nach 6 Takten wieder da, 35 A
    sleep_enable();  // sleep mode einschalten
    sleep_mode();    // in den sleep mode gehen
    sleep_disable(); // sleep mode ausschalten nach dem Erwachen
  }
  else {                                            // noch zu tun:
    if (actionPtr->time <= DebugSlowdown(micros() - timingZero)) {  // jetzt?
      SetPin(actionPtr->pin, actionPtr->value);     // Zustand [actionPtr] ausgeben.
      actionPtr++;                                  // actionPtr weiter,
#ifndef EOS_M100
      if (actionPtr->value == MOTOR_VAL) {
        //motorSuppress = 1; // s.u.
        EIFR = 1 << digitalPinToInterrupt(MOTOR);  // clear flag for pending interrupt, http://gammon.com.au/interrupts
        attachInterrupt(digitalPinToInterrupt(MOTOR), motor_interrupt, MOTOR_TRIGGER);
        actionPtr++;                                // actionPtr weiter,
      }
#endif
      if (actionPtr->value == BLINK_VAL) {
        BlinkLED();
        actionPtr++;                                // actionPtr weiter,
      }
      if (actionPtr->value == STOP_VAL) {         //  lschen wenn fertig
        actionPtr = 0;
      }
    }
  }
}

//////////////////////////////////////////////////////////////////////////////////////
// Interrupt-Handler
//
// a) Coil
//
void coil_interrupt() {
  if (digitalRead(COIL) == COIL_ACTIVE) {    // Coil aktiv
    triggerEvent = evCoilStart;     // Flag fr aktive Flanke
  }
  else {
    triggerEvent = evCoilStop;      // Ende
  }
  ToggleDebugLED();
}

// b) Motor: Triggert auf RISING. 
//  Hier unbesehen die Abschlussaktion anstoen. Falls gerade ausgelst wird, folgt
//   noch Coil. Dann wird diese Motor-Sequenz einfach berschrieben.   
// Alternative zur Erkennung der Auslsung: Motor-Sequenz nur nach evCoilStop
//  einleiten: if (previousEvent == evCoilStop)...
//  Beim Einschalten der Stromversorgung wird allerdings die Coil kurz aktiviert! Das msste
//   dann wieder in Setup() init. werden: Auslasszhler = 1
//
#ifndef EOS_M100
void motor_interrupt() {
#ifdef MOTOR_GATE
  s.o.;  Eleganter wre, den Interrupt temporr mit detachInterrupt(digitalPinToInterrupt(MOTOR)) stillzulegen
#endif  
#ifdef EOS_M3
  /*
  if (motorSuppress != 0) {
    // Seltsamerwese kommt bei der M3-Emulation sofort nach attachInterrupt ein
    // Interrupt, der unterdrckt werden muss!
    motorSuppress--;
    return;
  }
  */
  detachInterrupt(digitalPinToInterrupt(MOTOR));  // einmal reicht (Motor ist gepulst!)
#endif
  triggerEvent = evMotorOn;     // Flag fr Motor-An
  ToggleDebugLED();
}
#endif

// Wert val an Pin pin ausgeben
void SetPin(int pin, int val)
{
  digitalWrite(pin, val);
  ToggleDebugLED();
}
